@@ -44,6 +44,10 @@ gboolean udisks_daemon_util_setup_by_user (UDisksDaemon *daemon,
                                            UDisksObject *object,
                                            uid_t         user);
 
+gboolean udisks_daemon_util_on_same_seat (UDisksDaemon          *daemon,
+                                          UDisksObject          *object,
+                                          pid_t                  process);
+
 gboolean udisks_daemon_util_check_authorization_sync (UDisksDaemon          *daemon,
                                                       UDisksObject          *object,
                                                       const gchar           *action_id,
@@ -59,12 +63,31 @@ gboolean udisks_daemon_util_get_caller_uid_sync (UDisksDaemon            *daemon
                                                  gchar                  **out_user_name,
                                                  GError                 **error);
 
+gboolean udisks_daemon_util_get_caller_pid_sync (UDisksDaemon            *daemon,
+                                                 GDBusMethodInvocation   *invocation,
+                                                 GCancellable            *cancellable,
+                                                 pid_t                   *out_pid,
+                                                 GError                 **error);
+
 gpointer  udisks_daemon_util_dup_object (gpointer   interface_,
                                          GError   **error);
 
 gchar *udisks_daemon_util_escape (const gchar *str);
 gchar *udisks_daemon_util_escape_and_quote (const gchar *str);
 
+gchar *udisks_daemon_util_hexdump (gconstpointer data, gsize len);
+void udisks_daemon_util_hexdump_debug (gconstpointer data, gsize len);
+
+gboolean udisks_daemon_util_file_set_contents (const gchar  *filename,
+                                               const gchar  *contents,
+                                               gssize        contents_len,
+                                               gint          mode_for_new_file,
+                                               GError      **error);
+
+UDisksInhibitCookie *udisks_daemon_util_inhibit_system_sync   (const gchar          *reason);
+void                 udisks_daemon_util_uninhibit_system_sync (UDisksInhibitCookie  *cookie);
+
+
 G_END_DECLS
 
 #endif /* __UDISKS_DAEMON_UTIL_H__ */
