@@ -1103,9 +1103,9 @@ handle_mount (UDisksFilesystem       *filesystem,
   UDisksBlock *block;
   UDisksDaemon *daemon;
   UDisksCleanup *cleanup;
-  pid_t caller_pid;
   uid_t caller_uid;
   gid_t caller_gid;
+  pid_t caller_pid;
   const gchar * const *existing_mount_points;
   const gchar *probed_fs_usage;
   gchar *fs_type_to_use;
@@ -1216,10 +1216,10 @@ handle_mount (UDisksFilesystem       *filesystem,
           /* Translators: Shown in authentication dialog when the user
            * requests mounting a filesystem.
            *
-           * Do not translate $(udisks2.device), it's a placeholder and
+           * Do not translate $(drive), it's a placeholder and
            * will be replaced by the name of the drive/device in question
            */
-          message = N_("Authentication is required to mount $(udisks2.device)");
+          message = N_("Authentication is required to mount $(drive)");
           if (!udisks_daemon_util_setup_by_user (daemon, object, caller_uid))
             {
               if (udisks_block_get_hint_system (block))
@@ -1260,6 +1260,7 @@ handle_mount (UDisksFilesystem       *filesystem,
     mount_fstab_again:
       if (!udisks_daemon_launch_spawned_job_sync (daemon,
                                                   object,
+                                                  "filesystem-mount", caller_uid,
                                                   NULL,  /* GCancellable */
                                                   mount_fstab_as_root ? 0 : caller_uid, /* uid_t run_as_uid */
                                                   mount_fstab_as_root ? 0 : caller_uid, /* uid_t run_as_euid */
@@ -1281,13 +1282,13 @@ handle_mount (UDisksFilesystem       *filesystem,
                                                                  * user requests mounting a filesystem that is in
                                                                  * /etc/fstab file with the x-udisks-auth option.
                                                                  *
-                                                                 * Do not translate $(udisks2.device), it's a
+                                                                 * Do not translate $(drive), it's a
                                                                  * placeholder and will be replaced by the name of
                                                                  * the drive/device in question
                                                                  *
                                                                  * Do not translate /etc/fstab
                                                                  */
-                                                                N_("Authentication is required to mount $(udisks2.device) referenced in the /etc/fstab file"),
+                                                                N_("Authentication is required to mount $(drive) referenced in the /etc/fstab file"),
                                                                 invocation))
                 goto out;
               mount_fstab_as_root = TRUE;
@@ -1377,10 +1378,10 @@ handle_mount (UDisksFilesystem       *filesystem,
   /* Translators: Shown in authentication dialog when the user
    * requests mounting a filesystem.
    *
-   * Do not translate $(udisks2.device), it's a placeholder and
+   * Do not translate $(drive), it's a placeholder and
    * will be replaced by the name of the drive/device in question
    */
-  message = N_("Authentication is required to mount $(udisks2.device)");
+  message = N_("Authentication is required to mount $(drive)");
   if (!udisks_daemon_util_setup_by_user (daemon, object, caller_uid))
     {
       if (udisks_block_get_hint_system (block))
@@ -1436,6 +1437,7 @@ handle_mount (UDisksFilesystem       *filesystem,
   /* run mount(8) */
   if (!udisks_daemon_launch_spawned_job_sync (daemon,
                                               object,
+                                              "filesystem-mount", caller_uid,
                                               NULL, /* GCancellable */
                                               0,    /* uid_t run_as_uid */
                                               0,    /* uid_t run_as_euid */
@@ -1601,6 +1603,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
       /* right now -l is the only way to "force unmount" file systems... */
       if (!udisks_daemon_launch_spawned_job_sync (daemon,
                                                   object,
+                                                  "filesystem-unmount", caller_uid,
                                                   NULL, /* GCancellable */
                                                   unmount_fstab_as_root ? 0 : caller_uid, /* uid_t run_as_uid */
                                                   unmount_fstab_as_root ? 0 : caller_uid, /* uid_t run_as_euid */
@@ -1626,13 +1629,13 @@ handle_unmount (UDisksFilesystem       *filesystem,
                                                                  * user requests unmounting a filesystem that is in
                                                                  * /etc/fstab file with the x-udisks-auth option.
                                                                  *
-                                                                 * Do not translate $(udisks2.device), it's a
+                                                                 * Do not translate $(drive), it's a
                                                                  * placeholder and will be replaced by the name of
                                                                  * the drive/device in question
                                                                  *
                                                                  * Do not translate /etc/fstab
                                                                  */
-                                                                N_("Authentication is required to unmount $(udisks2.device) referenced in the /etc/fstab file"),
+                                                                N_("Authentication is required to unmount $(drive) referenced in the /etc/fstab file"),
                                                                 invocation))
                 goto out;
               unmount_fstab_as_root = TRUE;
@@ -1676,10 +1679,10 @@ handle_unmount (UDisksFilesystem       *filesystem,
        * requests unmounting a filesystem previously mounted by
        * another user.
        *
-       * Do not translate $(udisks2.device), it's a placeholder and
+       * Do not translate $(drive), it's a placeholder and
        * will be replaced by the name of the drive/device in question
        */
-      message = N_("Authentication is required to unmount $(udisks2.device) mounted by another user");
+      message = N_("Authentication is required to unmount $(drive) mounted by another user");
 
       if (!udisks_daemon_util_check_authorization_sync (daemon,
                                                         object,
@@ -1698,6 +1701,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
       escaped_mount_point = udisks_daemon_util_escape_and_quote (mount_point);
       rc = udisks_daemon_launch_spawned_job_sync (daemon,
                                                   object,
+                                                  "filesystem-unmount", caller_uid,
                                                   NULL, /* GCancellable */
                                                   0,    /* uid_t run_as_uid */
                                                   0,    /* uid_t run_as_euid */
@@ -1713,6 +1717,7 @@ handle_unmount (UDisksFilesystem       *filesystem,
       /* mount_point == NULL */
       rc = udisks_daemon_launch_spawned_job_sync (daemon,
                                                   object,
+                                                  "filesystem-unmount", caller_uid,
                                                   NULL, /* GCancellable */
                                                   0,    /* uid_t run_as_uid */
                                                   0,    /* uid_t run_as_euid */
@@ -1795,6 +1800,8 @@ handle_set_label (UDisksFilesystem       *filesystem,
   UDisksBaseJob *job;
   const gchar *action_id;
   const gchar *message;
+  uid_t caller_uid;
+  gid_t caller_gid;
   pid_t caller_pid;
   gchar *command;
   gchar *tmp;
@@ -1827,6 +1834,20 @@ handle_set_label (UDisksFilesystem       *filesystem,
       goto out;
     }
 
+  error = NULL;
+  if (!udisks_daemon_util_get_caller_uid_sync (daemon,
+                                               invocation,
+                                               NULL /* GCancellable */,
+                                               &caller_uid,
+                                               &caller_gid,
+                                               NULL,
+                                               &error))
+    {
+      g_dbus_method_invocation_return_gerror (invocation, error);
+      g_error_free (error);
+      goto out;
+    }
+
   probed_fs_usage = udisks_block_get_id_usage (block);
   probed_fs_type = udisks_block_get_id_type (block);
 
@@ -1857,15 +1878,17 @@ handle_set_label (UDisksFilesystem       *filesystem,
    * question in this case, check in advance */
   if (g_strcmp0 (probed_fs_type, "vfat") == 0)
     {
-      for (tmp = "\"*/:<>?\\|"; *tmp; ++tmp)
+      const gchar *forbidden = "\"*/:<>?\\|";
+      guint n;
+      for (n = 0; forbidden[n] != 0; n++)
         {
-          if (strchr (label, *tmp) != NULL)
+          if (strchr (label, forbidden[n]) != NULL)
             {
               g_dbus_method_invocation_return_error (invocation,
                                                      UDISKS_ERROR,
                                                      UDISKS_ERROR_NOT_SUPPORTED,
                                                      "character '%c' not supported in VFAT labels",
-                                                     *tmp);
+                                                     forbidden[n]);
                goto out;
             }
         }
@@ -1894,10 +1917,10 @@ handle_set_label (UDisksFilesystem       *filesystem,
   /* Translators: Shown in authentication dialog when the user
    * requests changing the filesystem label.
    *
-   * Do not translate $(udisks2.device), it's a placeholder and
+   * Do not translate $(drive), it's a placeholder and
    * will be replaced by the name of the drive/device in question
    */
-  message = N_("Authentication is required to change the filesystem label on $(udisks2.device)");
+  message = N_("Authentication is required to change the filesystem label on $(drive)");
   if (udisks_block_get_hint_system (block))
     {
       action_id = "org.freedesktop.udisks2.modify-device-system";
@@ -1931,6 +1954,7 @@ handle_set_label (UDisksFilesystem       *filesystem,
 
   job = udisks_daemon_launch_spawned_job (daemon,
                                           object,
+                                          "filesystem-modify", caller_uid,
                                           NULL, /* cancellable */
                                           0,    /* uid_t run_as_uid */
                                           0,    /* uid_t run_as_euid */
